==> eg_particle_interactions-v0.8.1-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/particles/TerrainParticle.java <==
package games.enchanted.eg_particle_interactions.common.mixin.particles;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import games.enchanted.eg_particle_interactions.common.config.ConfigHandler;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_1058;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.class_4003;
import net.minecraft.class_638;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(net.minecraft.class_727.class)
public abstract class TerrainParticle extends class_4003 {
    @Unique private static final float MIN_UV = 0.0000001f;
    @Unique private float block_place_particle$quadSizePixels = 1;

    @Mutable @Shadow @Final private float uo;
    @Mutable @Shadow @Final private float vo;

    protected TerrainParticle(class_638 level, double x, double y, double z) {
        super(level, x, y, z);
    }

    @Unique
    private void block_place_particle$recalculatePixelQuadSizes() {
        this.block_place_particle$quadSizePixels =
            this.field_17867 <= 0.04 ?
                (float) 1 / this.field_17886.method_45851().method_45807() :
                MathHelpers.ceilWithResolution(this.field_17867 + 0.0625, this.field_17886.method_45851().method_45807());

        if(this.uo + this.block_place_particle$quadSizePixels > 1) this.uo = 1 - this.block_place_particle$quadSizePixels;
        if(this.vo + this.block_place_particle$quadSizePixels > 1) this.vo = 1 - this.block_place_particle$quadSizePixels;
    }

    @Inject(
        at = @At("TAIL"),
        method = "<init>(Lnet/minecraft/client/multiplayer/ClientLevel;DDDDDDLnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/core/BlockPos;)V"
    )
    protected void terrainParticleInit(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_2680 state, class_2338 pos, CallbackInfo ci) {
        if(ConfigHandler.general_pixelConsistentTerrainParticles) {
            this.uo = (float) MathHelpers.randomBetween(0, this.field_17886.method_45851().method_45807()) / this.field_17886.method_45851().method_45807();
            this.vo = (float) MathHelpers.randomBetween(0, this.field_17886.method_45851().method_45815()) / this.field_17886.method_45851().method_45815();

            this.block_place_particle$recalculatePixelQuadSizes();
        }
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getU(F)F"),
        method = "getU0"
    )
    protected float alignU0(class_1058 instance, float u, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, u);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_17886.method_4580(this.uo) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getU(F)F"),
        method = "getU1"
    )
    protected float alignU1(class_1058 instance, float u, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, u);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_17886.method_4580(this.uo + this.block_place_particle$quadSizePixels) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getV(F)F"),
        method = "getV0"
    )
    protected float alignV0(class_1058 instance, float v, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, v);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_17886.method_4570(this.vo) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getV(F)F"),
        method = "getV1"
    )
    protected float alignV1(class_1058 instance, float v, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, v);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_17886.method_4570(this.vo + this.block_place_particle$quadSizePixels) + MIN_UV;
    }
}

==> eg_particle_interactions-v0.8.3-fabric-mc1.21.10-sources.jar: games/enchanted/eg_particle_interactions/common/mixin/particles/TerrainParticle.java <==
package games.enchanted.eg_particle_interactions.common.mixin.particles;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import games.enchanted.eg_particle_interactions.common.config.ConfigHandler;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_1058;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.class_3940;
import net.minecraft.class_638;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(net.minecraft.class_727.class)
public abstract class TerrainParticle
    //? if minecraft: <= 1.21.8 {
    /*extends TextureSheetParticle
    *///?} else {
    extends class_3940
    //?}
{
    @Unique private static final float MIN_UV = 0.0000001f;
    @Unique private float block_place_particle$quadSizePixels = 1;

    @Mutable @Shadow @Final private float uo;
    @Mutable @Shadow @Final private float vo;

    //? if minecraft: <=1.21.8 {
    /*protected TerrainParticle(ClientLevel level, double x, double y, double z) {
        super(level, x, y, z);
    }
    *///?} else {
    protected TerrainParticle(class_638 level, double x, double y, double z, class_1058 sprite) {
        super(level, x, y, z, sprite);
    }
    //?}

    @Unique
    private void block_place_particle$recalculatePixelQuadSizes() {
        this.block_place_particle$quadSizePixels =
            this.field_17867 <= 0.04 ?
                (float) 1 / this.field_62632.method_45851().method_45807() :
                MathHelpers.ceilWithResolution(this.field_17867 + 0.0625, this.field_62632.method_45851().method_45807());

        if(this.uo + this.block_place_particle$quadSizePixels > 1) this.uo = 1 - this.block_place_particle$quadSizePixels;
        if(this.vo + this.block_place_particle$quadSizePixels > 1) this.vo = 1 - this.block_place_particle$quadSizePixels;
    }

    @Inject(
        at = @At("TAIL"),
        method = "<init>(Lnet/minecraft/client/multiplayer/ClientLevel;DDDDDDLnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/core/BlockPos;)V"
    )
    protected void terrainParticleInit(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_2680 state, class_2338 pos, CallbackInfo ci) {
        if(ConfigHandler.general_pixelConsistentTerrainParticles) {
            this.uo = (float) MathHelpers.randomBetween(0, this.field_62632.method_45851().method_45807()) / this.field_62632.method_45851().method_45807();
            this.vo = (float) MathHelpers.randomBetween(0, this.field_62632.method_45851().method_45815()) / this.field_62632.method_45851().method_45815();

            this.block_place_particle$recalculatePixelQuadSizes();
        }
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getU(F)F"),
        method = "getU0"
    )
    protected float alignU0(class_1058 instance, float u, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, u);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_62632.method_4580(this.uo) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getU(F)F"),
        method = "getU1"
    )
    protected float alignU1(class_1058 instance, float u, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, u);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_62632.method_4580(this.uo + this.block_place_particle$quadSizePixels) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getV(F)F"),
        method = "getV0"
    )
    protected float alignV0(class_1058 instance, float v, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, v);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_62632.method_4570(this.vo) + MIN_UV;
    }

    @WrapOperation(
        at = @At(value = "INVOKE", target = "Lnet/minecraft/client/renderer/texture/TextureAtlasSprite;getV(F)F"),
        method = "getV1"
    )
    protected float alignV1(class_1058 instance, float v, Operation<Float> original) {
        if(!ConfigHandler.general_pixelConsistentTerrainParticles) {
            return original.call(instance, v);
        }
        this.block_place_particle$recalculatePixelQuadSizes();
        return this.field_62632.method_4570(this.vo + this.block_place_particle$quadSizePixels) + MIN_UV;
    }
}
